<?php
/**
 * This file contains the Profile class.
 *
 * @author Stefan Winter <stefan.winter@restena.lu>
 * @author Tomasz Wolniewicz <twoln@umk.pl>
 *
 * @package Developer
 *
 */

require_once('IdP.php');

/**
 * This class represents an EAP Profile.
 * Profiles can inherit attributes from their IdP, if the IdP has some. Otherwise,
 * one can set attribute in the Profile directly. If there is a conflict between
 * IdP-wide and Profile-wide attributes, the more specific ones (i.e. Profile) win.
 *
 * @author Stefan Winter <stefan.winter@restena.lu>
 * @author Tomasz Wolniewicz <twoln@umk.pl>
 *
 * @license http://ticker.eduroam.lu/cat/LICENSE
 *
 * @package Developer
 */
class Profile {

    public function  __construct($p_id) {
        $this->identifier = $p_id;
        if(! isset($this->name) || ! isset($this->institution)) {
            DBConnection::handle();
            $q = "SELECT name, inst_id, realm, use_anon_outer FROM profile WHERE profile_id = $p_id";
            debug(5,$q."\n");
            $profile = mysql_query($q);
            if(! $profile)
                return(FALSE);
            $a = mysql_fetch_object($profile);
            $this->name = $a->name;
            $this->institution = $a->inst_id;
            $this->realm = $a->realm;
            $this->use_anon_outer = $a->use_anon_outer;
        }
    }

    public function flushAttributes($eap_type) {
        commitFlushAttributes(beginFlushAttributes($eap_type));
    }

    /**
     * deletes all attributes in this profile except the _file ones, these are reported as array
     *
     * @return array list of row id's of file-based attributes which weren't deleted
     */
    public function beginFlushAttributes($eap_type) {
        $q_delete = "DELETE FROM profile_option WHERE profile_id = $this->identifier AND eap_method_id = $eap_type AND option_name NOT LIKE '%_file'";
        debug(5,$q_delete."\n");
        $exec_q = mysql_query($q_delete);
        $q_remaining = "SELECT row FROM profile_option WHERE profile_id = $this->identifier AND eap_method_id = $eap_type";
        debug(5,$q_remaining."\n");
        $exec_q = mysql_query($q_remaining);
        $return_array = array();
        while ($a = mysql_fetch_object($exec_q))
            $return_array[$a->row] = "KILLME";
        return $return_array;
    }
    /**
     * after a beginFlushAttributes, deletes all attributes which are in the tobedeleted array
     *
     * @param array $tobedeleted array of database rows which are to be deleted
     */
    public function commitFlushAttributes($tobedeleted) {
        foreach ($tobedeleted as $row => $unimportant) {
            $q_delete = "DELETE FROM profile_option WHERE profile_id = $this->identifier AND row = $row";
            debug(5,$q_delete."\n");
            $exec_q = mysql_query($q_delete);
        }
    }

    /**
     * registers that a combination of device type and EAP method in this profile should NOT be served by CAT
     * A redirect URL needs to be given to send the user to
     *
     * @param EAP $eaptype if omitted, will blacklist ALL EAP types for this device
     * @param string $device if omitted, will blacklist the EAP type for all devices
     * @param string $targetURL
     *
     * @return bool whether or not the operation succeeded. The only reason for failure is if a superset rule exists already.
     */
    public function registerBlacklist($targetURL, $eaptype = 0, $device = 0) {

        // unregister all blacklists which are more specific than the requested one

        $this->unregisterBlacklist($eaptype, $device);

        if (($device === 0) && ($eaptype === 0)) { // just massage input value
            $device = "NULL";
        }
        elseif ($device === 0) { // blacklist all devices, for one specific EAP type
            // see if a superset blacklist rule exists, refuse operation if so
            if ($this->isBlacklisted($eaptype, $device)) return FALSE;
            $device = "NULL";
        }
        elseif ($eaptype === 0) { // blacklist all EAP types for one device
            // see if a superset blacklist rule exists, refuse operation if so
            if ($this->isBlacklisted($eaptype, $device)) return FALSE;
            $device = "'$device'";
        }
        else { // just one specific blacklist item. See if it's already covered by a superset
            if ($this->isBlacklisted($eaptype, $device)) return FALSE;
            $device = "'$device'";
        }

        $q = "INSERT INTO installer_blacklist (profile_id, eap_method_id, device_name, redirection_target)
                                    VALUES($this->identifier, $eaptype, $device, '$targetURL')";

        debug(5,$q."\n");
        $exec_q = mysql_query($q);
        return TRUE;
    }

    /**
     * Deletes a previously set blacklist rule for EAP types and/or devices in this profile, including all more specific rules
     * @param EAP $eaptype if omitted, will lift blacklist for ALL EAP types for this device
     * @param string $device if omitted, will lift blacklist of the EAP type for all devices
     */
    public function unregisterBlacklist($eaptype = 0, $device = 0) {
        if (($device === 0) && ($eaptype === 0)) { // delete all redirects
            $delete = "DELETE FROM installer_blacklist WHERE profile_id = $this->identifier";
            debug(5,$delete."\n");
            $exec_q = mysql_query($delete);
        }
        elseif ($device === 0) { // delete all blacklists for one specific EAP type
            $delete = "DELETE FROM installer_blacklist WHERE profile_id = $this->identifier AND eap_method_id = $eaptype";
            debug(5,$delete."\n");
            $exec_q = mysql_query($delete);
        }
        elseif ($eaptype === 0) { // delete all blacklists for one device
            $delete = "DELETE FROM installer_blacklist WHERE profile_id = $this->identifier AND device_name = '$device'";
            debug(5,$delete."\n");
            $exec_q = mysql_query($delete);
        }
        else { // delete just one specific blacklist item
            $delete = "DELETE FROM installer_blacklist WHERE profile_id = $this->identifier AND device_name = '$device' AND eap_method_id = $eaptype";
            debug(5,$delete."\n");
            $exec_q = mysql_query($delete);
        }
    }

    /**
     * Checks whether a combination of EAP type and device should be served, or rather redirected
     * @param <type> $eaptype
     * @param <type> $device
     *
     * @return mixed FALSE if the combination is not blacklisted, a string with the redirection URL if so
     */
    public function isBlacklisted($eaptype, $device) {
        $q = "SELECT redirection_target FROM installer_blacklist WHERE profile_id = $this->identifier AND (eap_method_id = $eaptype OR eap_method_id = 0) AND (device_name = '$device' OR device_name IS NULL)";

        debug(5,$q."\n");
        $exec_q = mysql_query($q);

        if ($result = mysql_fetch_object($exec_q))
            return $result->redirection_target;
        else
            return FALSE;
    }

    /**
     * adds an attribute to this profile
     *
     * @param string $attr_name
     * @param string $attr_value
     * @param int $eap_type
     */
    public function addAttribute($attr_name,$attr_value, $eap_type) {
        $q_insert = "INSERT INTO profile_option (profile_id, option_name, option_value, eap_method_id) VALUES("
                .$this->identifier.", '"
                .$attr_name."', '"
                .$attr_value
                ."', $eap_type)";
        $exec_q = mysql_query($q_insert);
    }

    public function setName($name) {
        $this->name = $name;
        $q = "UPDATE profile SET name = '$name' WHERE profile_id = $this->identifier";
        $exec_q = mysql_query($q);
    }

    /**
     * register new supported EAP method for this profile
     *
     * @param array $type The EAP Type, as defined in class EAP
     * @param int $preference preference of this EAP Type. If a preference value is re-used, the order of EAP types of the same preference level is undefined.
     *
     */
    public function addSupportedEapMethod($type, $preference) {
        $q = "SELECT eap_method_id from eap_method WHERE php_serialized = '".serialize($type)."'";
        $type_id = mysql_query($q);
        if ($a = mysql_fetch_object($type_id)) {
            $insert = "INSERT INTO supported_eap (profile_id, eap_method_id, preference) VALUES ("
                    .$this->identifier.", "
                    .$a->eap_method_id.", "
                    .$preference.")";
            $exec_insert = mysql_query($insert);

        } else { // that EAP type is unknown
            exit(1);
        }
    }

    /**
     * deletes the profile from database and uninstantiates itself
     *
     */

    public function destroy() {
        $q_delete_options = "DELETE FROM profile_option WHERE profile_id = $this->identifier";
        debug(5,$q_delete_options."\n");
        $q_delete_supported_eap = "DELETE FROM supported_eap WHERE profile_id = $this->identifier";
        debug(5,$q_delete_supported_eap."\n");
        $q_delete_profile = "DELETE FROM profile WHERE profile_id = $this->identifier";
        debug(5,$q_delete_profile."\n");
        $exec_q1 = mysql_query($q_delete_options);
        $exec_q2 = mysql_query($q_delete_supported_eap);
        $exec_q3 = mysql_query($q_delete_profile);
        unset($this);
    }

    /** removes all supported EAP methods
     *
     */

    public function flushSupportedEapMethods() {
        $q = "DELETE FROM supported_eap WHERE profile_id = $this->identifier";
        debug(5,$q."\n");
        $exec_delete = mysql_query($q);
    }


    /** toggle anonID support
     *
     * @param boolean $shallwe
     * @param string $realm
     *
     */

    public function setAnonymousIDSupport ($shallwe, $realm = 0) {
        // realm is not evaluated if $shallwe is false
        DBConnection::handle();
        $q;
        if ($shallwe == true && $realm !== 0) { // support anon ID
            $q = "UPDATE profile SET use_anon_outer = 1,realm = '".$realm."' WHERE profile_id = $this->identifier";
        } else {
            $q = "UPDATE profile SET use_anon_outer = 0 WHERE profile_id = $this->identifier";
        };

        $updated = mysql_query($q);
    }

    public function setRealm ($realm) {
        DBConnection::handle();
        $q = "UPDATE profile SET realm = '".$realm."' WHERE profile_id = $this->identifier";
        $updated = mysql_query($q);
    }

    public function getRealm() {
        DBConnection::handle();
        $q = "SELECT realm FROM profile WHERE profile_id = $this->identifier";
        $db_realm = mysql_query($q);
        if ($result_realm = mysql_fetch_object($db_realm))
            return $result_realm->realm;
        else
            return FALSE;
    }

    /** returns FALSE if anonymous outer ID support if disabled, or the realm if it is enabled */

    public function getAnonymousIDSupport () {
        $q = "SELECT use_anon_outer, realm FROM profile WHERE profile_id = $this->identifier";
        DBConnection::handle();
        $anon = mysql_query($q);
        if ($anonsupport = mysql_fetch_object($anon))
            if ($anonsupport->use_anon_outer == 1)
                return $anonsupport->realm;
            else
                return FALSE;
    }

    public function getEapMethodsinOrderOfPreference() {
        DBConnection::handle();
        $q = "SELECT meth.php_serialized FROM supported_eap supp,eap_method meth
               WHERE supp.profile_id = $this->identifier AND supp.eap_method_id = meth.eap_method_id
               ORDER by preference";
        debug(5,$q."\n");
        $eap_m = mysql_query($q);
        $returnarray = array();
        while ($eap = (mysql_fetch_object($eap_m)))
            $returnarray[] = unserialize($eap->php_serialized);
        debug(4, "Looks like this profile supports the following EAP types: ");
        debug(4,$returnarray);
        return($returnarray);
    }

    /** getAttibutes() returnes an array of attributes, including options (as an array)
     * if no argument is given, only the profile-wide options are returned
     *
     * @return array
     */
    public function getAttributes($eapmethod = 0) {

        DBConnection::handle();

        $eapspecificoptions = array();

        if ($eapmethod) {
            $serialized_name = serialize($eapmethod);

            $q = "SELECT option_name, option_value, row FROM profile_option
                                                JOIN eap_method ON profile_option.eap_method_id = eap_method.eap_method_id
                   WHERE profile_id = $this->identifier AND eap_method.php_serialized = '$serialized_name' ORDER BY option_name";
            debug(5,$q."\n");

            $EAPAttributes = mysql_query($q);

            while($a = mysql_fetch_object($EAPAttributes)) {
                $eapspecificoptions[] = array( "name" => $a->option_name, "value" => $a->option_value, "level" => "Method", "row" => $a->row);
            }

        }

        $q = "SELECT option_name, option_value, row FROM profile_option
                                                WHERE profile_id = $this->identifier AND eap_method_id = 0 ORDER BY option_name";

        debug(5,$q."\n");
        $ProfileAttributes = mysql_query($q);

        $profileoptions = array();

        while($a = mysql_fetch_object($ProfileAttributes)) {
            $profileoptions[] = array( "name" => $a->option_name, "value" => $a->option_value, "level" => "Profile", "row" => $a->row);
        }



        // now, fetch IdP-wide attributes and see if there some IdP-wide which aren't set on profile level yet

        $idp = new IdP($this->institution);
        $idpoptions = $idp->getAttributes();



        // now, merge all the options. If an option is defined in a more specific level, ignore the less-specific ones.
        // this is a O(n+m) operation :-(

        // see which options are already set on per-EAP level

        $consolidatedoptions = array();

        foreach ($eapspecificoptions as $optiontuple)
            if ( ! in_array($optiontuple["name"],$consolidatedoptions))
                $consolidatedoptions[] = $optiontuple["name"];

        // if we find a new option in the per-profile ones, add it

        foreach ($profileoptions as $profileitem)
            if ( ! in_array($profileitem["name"], $consolidatedoptions) ) {
                $eapspecificoptions[] = $profileitem;
        };

        // see which options are now set on per-profile level

        $consolidatedoptions = array();
        foreach ($eapspecificoptions as $optiontuple)
            if ( ! in_array($optiontuple["name"],$consolidatedoptions))
                $consolidatedoptions[] = $optiontuple["name"];

        // now, merge with the IdP-wide options

        foreach ($idpoptions as $idpitem)
            if ( ! in_array($idpitem["name"],$consolidatedoptions))
                $eapspecificoptions[] = $idpitem;

        $eapspecificoptions[] = array("name" => "internal:profile_name", "value" => $this->name, "level" => "Profile", "row" => 0);
        $eapspecificoptions[] = array("name" => "internal:realm", "value" => $this->realm, "level" => "Profile", "row" => 0);
        $eapspecificoptions[] = array("name" => "internal:use_anon_outer", "value" => $this->use_anon_outer, "level" => "Profile", "row" => 0);

        return $eapspecificoptions;

    }

    public function isEapTypeDefinitionComplete($eaptype) {
        // TLS, TTLS, PEAP outer phase need a CA certficate and a Server Name
        if ( $eaptype["OUTER"] == PEAP || $eaptype["OUTER"] == TLS || $eaptype["OUTER"] == TTLS ) {
            $ca_present = false;
            $cn_present = false;

            $options = $this->getAttributes($eaptype);

            foreach ($options as $option)
                if ( $option["name"] == "eap:ca_file" ) $ca_present = true;

            foreach ($options as $option)
                if ( $option["name"] == "eap:server_name" ) $cn_present = true;

            return $ca_present && $cn_present;
        }

        // no idea what FAST really needs. Let's say it's alright.

        if ( $eaptype["OUTER"] == FAST) return false;

        // we have no idea; let's say false

        return false;
    }

    public function getCollapsedAttributes () {
        $attr = $this->getAttributes(func_get_args());
        $out = array();
        $temp=array();
        $temp1 = array();
        foreach ($attr as $b) {
            $name = $b['name'];
            $temp1[] = $name;
            $level = $b['level'];
            $value = $b['value'];
            if(!isset($temp[$name][$level]))
                $temp[$name][$level] = array();
            $temp[$name][$level][] = $value;
        }
        foreach ($temp1 as $name) {
            if(isset($temp[$name]['Method']))
                $out[$name] = $temp[$name]['Method'];
            elseif(isset($temp[$name]['Profile']))
                $out[$name] = $temp[$name]['Profile'];
            else
                $out[$name] = $temp[$name]['IdP'];
        }
        return($out);
    }


    private function addCA ($blob) {
        $cadata;
        if (preg_match("/:\/\//", $blob)) {
            # we got a URL, download it
            $download = fopen($blob,"rb");
            $cadata = stream_get_contents($download);
        } else {
            # no URL... treat it as a binary blob
            $cadata = $blob;
        }
        if( substr($cadata,0,27) == "-----BEGIN CERTIFICATE-----" ) {
            $ca_pem = $cadata;
            $ca_der = X509::pem2der($ca_pem);
        } else {
            $ca_der = $cadata;
            $ca_pem =  X509::der2pem($cadata);
        }

        # check that the certificate is OK
        $myca = openssl_x509_read($ca_pem);
        if ($myca == FALSE) return FALSE;
        $md5 = openssl_digest($ca_der,'MD5');
        $sha1 = openssl_digest($ca_der,'SHA1');

        $this->CAs[] = array ("uuid" => X509::uuid(), "pem" => $ca_pem, "der" => $ca_der, "md5"=>$md5, "sha1"=>$sha1);
        $this->sha1[] = $sha1;
        return TRUE;
    }

    public function getCAs($certs) {
        $this->CAs = array();
        foreach($certs as $blob)
            $this->addCA($blob);
        return array('cert'=>$this->CAs,'sha1'=>$this->sha1);
    }

    public $identifier;
    public $name;
    public $institution;
    public $realm;
    public $use_anon_outer;

}

?>
