<?php
/**
 * This file contains the TestModule class
 *
 * This is a very basic example of using the CAT API.  
 *
 * The module contains two files
 * in the Files directory. They will illustrate the use of the {@link DeviceConfigTT::copyFile()} method.
 * One fille will be coppied without the name change, for the second we will provide a new name.
 *
 * The module will collect all certificate files stored in the database for a given profile and wiil copy them to the working directory.
 *
 * If, for the given profile, an information file is available, this too will be copied to the working directory.
 *
 * The installer will collect all available configuration attributes and save them to a file in the form of print_r output.
 *
 * Finally the installer will create a zip archive containing all above files and this file 
 * will be sent to the user as the configurator file.
 *  
 * @package ModuleWriting
 */

/**
 * this array holds the list of EAP methods supported by this device
 */

/**
 * 
 */
require_once('Device.php');

/**
 * This is the main implementation class of the module
 *
 * The name of the class must the the 'Device_' followed by the name of the module file
 * (without the '.php' extension), so in this case the file is "TestModule.php" and
 * the class is Device_TestModule.
 *
 * The class should only define one public method: writeInstaller.
 *
 * All other methods and properties should be private. This example sets zipInstaller method to protected, so that it can be seen in the documentation.
 *
 * @package ModuleWriting
 */
class Device_TestModule extends DeviceConfig {
   /**
    * Constructs a Device object.
    *
    * It is CRUTCIAL that the constructor sets $this->supportedEapMethods to an array of methods
    * available for the particular device.
    * @param string $device a pointer to a device module, which must
    * be an index of one of the devices defined in the {@link Devices}
    * array in {@link devices.php}.
    * @final not to be redefined
    */
    final public function __construct($device) {
      debug(4,"got device: $device\n");
      $this->supportedEapMethods  = array(EAP::$TLS, EAP::$PEAP_MSCHAP2, EAP::$TTLS_PAP);
      debug(4,"This device supports the following EAP methods: ");
      debug(4,$this->supportedEapMethods);
    }

  /**
   * prepare a zip archive containing files and settings which normally would be used inside the module to produce an installer
   *
   * {@source}
   */
   public function writeInstaller(Profile $profile) {
   /* run innitial setup
      this will:
       - create the temporary directory and save its path as $this->FPATH
       - process the CA certificates and store results in $this->attributes['internal:CAs'][0]
            $this->attributes['internal:CAs'][0] is an array of processed CA certificates
            a processed certifincate is an array 
               'pem' points to pem feromat certificate
               'der' points to der format certificate
               'md5' points to md5 fingerprint
               'sha1' points to sha1 fingerprint
               'name' points to the certificate subject
       - save the info_file (if exists) and put the name in $this->attributes['internal:info_file_name'][0]
   */
debug(4,"Test Module Installer start\n");
   // create certificate files and save their names in $CA_files arrary
     $CA_files = $this->saveCertificateFiles('der');
// tkip-profile is currently used only by Windows
//     $tkip_profile_name = 'eduroam (TKIP)';
  
    // copy a fixed file from the module Files directory
       if(! $this->copyFile('Module.howto'))
          debug(2, "copying of Module.howto failed\n");
    // copy a fixed file from the module Files directory and saveunde a different name
       if( ! $this->copyFile('test_file','copied_test_file'))
          debug(2, "copying of Module.howto to copied_test_file failed\n");
       $this->dumpAttibutes('profile_attributes');
    $installer_path =  $this->zipInstaller($this->attributes);
    return($installer_path);
   }

/**
  * zip files and return the archive name
  *
  * inline{@source}
  * return string
  */
  protected function zipInstaller($attr) {
    $e = preg_replace('/ +/','_',$attr['internal:inst_name'][0]).'.zip';
    $o = system('zip -q '.$e.' *');
    return $e;
  }

}
