<?php
require_once(dirname(dirname(dirname(__FILE__)))."/config/_config.php");
include_once("Helper.php");
include_once("IdP.php");
include_once("Profile.php");
include_once("DeviceFactory.php");
include_once("devices/devices.php");
include_once("inc/common.inc.php");

if (isset($_GET['lang'])) {
    $ourlocale = set_locale("web_admin",$_GET['lang']);
    $lang_string = "&lang=".$_GET['lang'];
}
else {
    $ourlocale = set_locale("web_admin");
    $lang_string="";
}
header("Content-Type:text/html;charset=utf-8");
?>
<!DOCTYPE html>
<html>
    <head>
        <meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
        <meta http-equiv="content-language" content="<?php echo $ourlocale;?>">
        <title><?php echo _("Device Compatibility matrix");?></title>
        <link rel='stylesheet' type='text/css' href='../resources/css/cat.css' />
    </head>
    <body>
        <?php
        include "inc/admin_header.php";
        $my_inst = new IdP($_GET['inst_id']);

        if (! $my_inst instanceof IdP) {
            echo "<p>"._("The page was called with a non-existing IdP identifier. This is not supposed to happen!")."</p>";
            exit(1);
        }

        $my_profile = new Profile($_GET['profile_id']);
        if (! $my_profile instanceof Profile) {
            echo "<p>"._("The page was called with a Profile identifier, but there was no such profile in this institution!")."</p>";
            exit(1);
        }
        $preflist = $my_profile->getEapMethodsinOrderOfPreference();
        ?>
        <h1><?php printf(_("Device compatiblity matrix for %s of %s "),$my_profile->name,$my_inst->name);?></h1>
        <table class="compatmatrix">
            <tr>
                <th><?php echo _("Device");?></th>

                <?php
                foreach ($preflist as $method) {
                    echo "<th>".display_name($method)."</th>";
                }
                ?>

            </tr>
            <?php
            // okay, input is valid. Now create a table: columns are the EAP types supported in the profile,
            // rows are known devices

            $devices = Devices::listDevices();

            foreach ($devices as $index => $description) {
                echo "<tr>";
                echo "<td>".$description['display']."<br/><form><button>"._("Redirect device for all EAP types")."</button></form></td>";
                $factory = new DeviceFactory($index);
                $defaultisset = FALSE;
                foreach ($preflist as $method) {
                    /* echo "<pre>Method:";
                print_r($method);
                echo "supported:";
                print_r($factory->device->supportedEapMethods);
                echo "</pre>"; */

                    if (in_array($method, $factory->device->supportedEapMethods)) {
                        if (!$my_profile->isEapTypeDefinitionComplete($method)) {
                            echo "<td><div class='warning'><strong>"._("Incomplete")."</strong></span><br/><button>"._("Redirect")."</button></form></td>";
                        }
                        elseif ($method === $preflist[0] || $defaultisset == FALSE) {
                            echo "<td><div class='acceptable'><strong>"._("Default")."</strong></span><br/><form><button>"._("Redirect")."</button></form><form><button>"._("Download")."</button></form></td>";
                            $defaultisset = TRUE;
                        }
                        else {
                            echo "<td><div class='secondary'>"._("Configured")."</span><br/><button>"._("Redirect")."</button></form></td>";
                        }

                    }
                    else
                        echo "<td><div class='notacceptable'>"._("Unsupported")."</span><br/><button>"._("Redirect")."</button></form></td>";
                }
                echo "</tr>";
            }
            ?>
            <tr>
                <td></td>

                <?php
                foreach ($preflist as $method) {
                    echo "<td style='border-top: 2px solid #000000;'><form><button>"._("Redirect EAP type for all devices")."</button></form></th>";
                }
                ?>

            </tr>
        </table>
        <p><strong>Legend:</strong></p>
        <table>
            <tr><td><div class="acceptable"><strong><?php echo _("Default");?></strong></div></td><td><?php echo _("means that the EAP type is properly configured, supported by the device, and will be made available on the download page");?></td></tr>
            <tr><td><div class="secondary"><?php echo _("Configured");?></div></td><td><?php echo _("means that the EAP type is properly configured, supported by the device, but will not be made available because a more preferable type is also available");?></td></tr>
            <tr><td><div class="warning"><strong><?php echo _("Incomplete");?></strong></div></td><td><?php echo _("means that the EAP type is not properly configured, so generation of profiles is not possible even though the device could handle that EAP type");?></td></tr>
            <tr><td><div class="notacceptable"><?php echo _("Unsupported");?></div></td><td><?php echo _("means the CAT does not support this combination of device and EAP type. If you know how to make this combination work, please let us know!");?></td></tr>
        </table>
        <?php
        $buttontext = _("Return to dashboard");
        echo "<form method='post' action='dashboard.php?inst_id=$my_inst->identifier$lang_string'><button type='submit' name='submitbutton' value='return'>".$buttontext."</button></form>";?>
    </body>
</html>
