<?php
require_once(dirname(dirname(dirname(__FILE__)))."/config/_config.php");
include_once("Helper.php");
include_once("Federation.php");
include_once("IdP.php");
include_once("Profile.php");

include_once("inc/common.inc.php");

if (isset($_GET['lang'])) {
    $ourlocale = set_locale("web_admin",$_GET['lang']);
    $lang_string = "&lang=".$_GET['lang'];
}
else {
    $ourlocale = set_locale("web_admin");
    $lang_string="";
}
header("Content-Type:text/html;charset=utf-8");
?>
<!DOCTYPE html>
<html>
    <head>
        <meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
        <meta http-equiv="content-language" content="<?php echo $ourlocale;?>">
        <title><?php echo _("eduroam CAT: IdP Dashboard")?></title>
        <link rel='stylesheet' type='text/css' href='../resources/css/cat.css' />
    </head>
    <body>
        <?php
        include "inc/admin_header.php";
        // let's check if the inst handle actually exists in the DB
        $my_inst = new IdP($_GET['inst_id']);

        if (! $my_inst instanceof IdP) {
            echo "<p>"._("The page was called with a non-existing IdP identifier. This is not supposed to happen!")."</p>";
            exit(1);
        }
        // Sanity check complete. Show what we know about this IdP.
        $idpoptions = $my_inst->getAttributes();
        ?>
        <h1><?php echo _("Identity Provider Overview");?></h1>
        <div>
            <h2><?php echo _("IdP-wide settings");?></h2>
            <div class="infobox"><h2><?php echo _("General Institution Details");?></h2>
                <table>
                    <tr>
                        <td>
                            <?php echo _("Country:");?>
                        </td>
                        <td>
                            <strong><?php echo Federation::name($my_inst->federation);?></strong>
                        </td>
                    </tr>
                    <tr>
                        <td>
                            <?php echo _("Institution name:");?>
                        </td>
                        <td>
                            <strong><?php echo $my_inst->name;?></strong>
                            <?php echo infoblock($idpoptions, "general", "IdP");?>
                        </td>
                    </tr>
                </table>
            </div>
            <div class="infobox">
                <h2><?php echo _("Global Helpdesk Details");?></h2>
                <table>
                    <?php echo infoblock($idpoptions, "support", "IdP");?>
                </table>
            </div>
            <div class='infobox'>
                <h2><?php echo _("Global EAP Options");?></h2>
                <table>
                    <?php echo infoblock($idpoptions, "eap", "IdP");?>
                </table>
            </div>
        </div>
        <?php
        echo "<form action='idp_wizard.php?inst_id=$my_inst->identifier$lang_string' method='post'><button type='submit' name='submitbutton' value='editidp'>"._("Edit IdP-wide settings")."</button></form>";

        $numprofiles = count ( $my_inst->listProfiles() );

        if ( $numprofiles == 0 ) {
            // no profiles yet.
            echo _("There are not yet any profiles for your institution.");
        } else {
            echo "<h2>"._("Profiles for this institution")."</h2>";
            foreach ( $my_inst->listProfiles() as $profile_list ) {
                // see if there are any profile-level overrides
                $attribs = $profile_list->getAttributes();

                echo "<div class='profilebox'><h2>".sprintf(_("Profile: %s"),$profile_list->name)."</h2>";

                $has_overrides = FALSE;
                foreach ($attribs as $attrib)
                    if ($attrib['level'] == "Profile" && ! preg_match("/^internal:/",$attrib['name']))
                        $has_overrides = TRUE;
                
                echo "<div style='margin-bottom:60px'>"._("<strong>EAP Types</strong> (in order of preference):")."<br/>";
                $typelist = $profile_list->getEapMethodsinOrderOfPreference();
                $allcomplete = TRUE;
                foreach ( $typelist as $eaptype ) {
                    echo display_name($eaptype);
                    if ($profile_list->isEapTypeDefinitionComplete($eaptype)) {
                        echo " <div class='acceptable'>"._("OK")."</div>";
                    } else {
                        echo " <div class='notacceptable'>"._("Information needed!")."</div>";
                        $allcomplete = FALSE;
                    };
                    $eapattribs = $profile_list->getAttributes($eaptype);
                    foreach ($attribs as $attrib)
                        if ($attrib['level'] == "Method" && ! preg_match("/^internal:/",$attrib['name']))
                            echo "<img src='../resources/images/icons/Letter-E-blue-icon.png' alt='"._("Option override on EAP Method level is in effect")."'>";
                    echo "<br/>";
                }
                echo "<div style='position:absolute; top:5px; right:5px'>";
                if ($has_overrides) echo "<img src='../resources/images/icons/Letter-P-blue-icon.png' alt='"._("Option override on Profile level is in effect.")."' title='"._("Option override on Profile level is in effect.")."'/>";
                if (!$allcomplete) echo "<img src='../resources/images/icons/Exclamation-orange-icon.png' alt='"._("The information in this Profile is incomplete.")."' title='"._("The information in this Profile is incomplete.")."'/>";
                echo "</div>";
                if ( array_search(EAP::$TTLS_PAP, $typelist) !== FALSE && array_search(EAP::$TTLS_GTC, $typelist) === FALSE && array_search(EAP::$PEAP_MSCHAP2,$typelist) === FALSE && array_search(EAP::$TTLS_MSCHAP2,$typelist) === FALSE )
                /// Hmmm... IdP Supports TTLS-PAP, but not TTLS-GTC nor anything based on MSCHAPv2. That locks out Symbian users; and is easy to circumvent. Tell the admin...
                    echo "<p>"._("Read this <a href='gtc-support.html'>tip</a>.")."</p>";

                echo "</div>";
                $has_eaptypes = count($profile_list->getEapMethodsInOrderOfPreference());
                $has_realm = $profile_list->getRealm();
                echo "<div class='profilemodulebuttons'>
                          <form action='check-dynamic.php?inst_id=$my_inst->identifier&profile_id=".$profile_list->identifier."$lang_string' method='post'>
                              <button type='submit' name='profile_action' value='check' ".($has_realm ? "" : "disabled='disabled' title='"._("The profile's realm can only be checked if you configure the realm!")."'").">
                                  "._("Check realm reachability")."
                              </button>
                          </form>
                          <form action='compatibility_matrix.php?inst_id=$my_inst->identifier&profile_id=".$profile_list->identifier."$lang_string' method='post'>
                              <button type='submit' name='profile_action' value='check' ".($has_eaptypes ? "" : "disabled='disabled'  title='"._("You have not configured any supported EAP types!")."'").">
                                  "._("Device Compatiblity Matrix")."
                              </button>
                          </form>
                      </div>
                      <div class='buttongroupprofilebox'>
                          <form action='profile_wizard.php?inst_id=$my_inst->identifier&profile_id=".$profile_list->identifier."$lang_string' method='post'>
                               <hr/>
                               <button type='submit' name='profile_action' value='edit'>"._("Edit")."</button>
                               <button type='submit' name='profile_action' value='delete' onclick=\"return confirm('".sprintf(_("Do you really want to delete the profile %s?"),$profile_list->name)."')\">
                                   "._("Delete")."
                               </button>
                           </form>
                      </div>";
                echo "</div>";
            }
        }

        echo "<form action='profile_wizard.php?inst_id=".$my_inst->identifier.$lang_string
                ."' method='post'><div><button type='submit' name='profile_action' value='new'>"._("Add new profile ...")."</button></div></form>";
        ?>
        <?php include "inc/admin_footer.php"; ?>
    </body>
</html>
