<?php
require_once(dirname(dirname(dirname(__FILE__)))."/config/_config.php");
include_once("Federation.php");
include_once("IdP.php");
include_once("Helper.php");
include_once("inc/common.inc.php");

if (isset($_GET['lang'])) {
    $ourlocale = set_locale("web_admin",$_GET['lang']);
    $lang_string = "&lang=".$_GET['lang'];
}
else {
    $ourlocale = set_locale("web_admin");
    $lang_string="";
}

// let's check if the inst handle actually exists in the DB

$my_inst = new IdP($_GET['inst_id']);

if (! $my_inst instanceof IdP) {
    echo "<p>"._("The page was called with a non-existing IdP identifier. This is not supposed to happen!")."</p>";
    exit(1);
}

// how do we determine if we should go into wizard mode? Well if there are no options
// whatsoever, assume this is new.

$idpoptions = $my_inst->getAttributes();

if (isset($_GET['wizard']) && $_GET['wizard'] == "true") {
    $wizard_style = TRUE;
    $edit_mode = FALSE;
} else {
    $wizard_style = FALSE;
    $edit_mode = TRUE;
}

// do we have POST data? Then we should continue to the next step.

if ( isset( $_POST['option']) && isset ($_POST['value']) ) { // here we go
    $remaining_attribs = $my_inst->beginflushAttributes();
    $killlist = processSubmittedFields($my_inst,$remaining_attribs);
    // print_r($killlist);
    $my_inst->commitFlushAttributes($killlist);

    if (!$wizard_style)
        header("Location: dashboard.php?inst_id=$my_inst->identifier$lang_string");
    else
        header("Location: profile_wizard.php?inst_id=$my_inst->identifier$lang_string");
};
header("Content-Type:text/html;charset=utf-8");
?>
<!DOCTYPE html>
<html>
    <head>
        <meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
        <meta http-equiv="content-language" content="<?php echo $ourlocale;?>">
        <title>
            <?php
            if ($wizard_style)
                echo _("eduroam CAT: IdP enrollment wizard (step 2)");
            else
                printf(_("eduroam CAT: Editing IdP '%s'"),$my_inst->name);
            ?>
        </title>
        <link rel="stylesheet" type="text/css" href="../resources/css/cat.css" />
        <script src="js/option_expand.js" type="text/javascript"></script>
        <?php
        $part = "head";
        $inst_country = $my_inst->federation;
        $inst_name = $my_inst->name;
        $additional = FALSE;
        foreach ($idpoptions as $optionname => $optionvalue)
            if ($optionvalue['name'] == "general:geo_coordinates") $additional = TRUE;
        include "inc/geo_widget.php";
        ?>
    </head>
    <body onload='load()'>

        <?php include "inc/admin_header.php";?>
        <h1>
            <?php
            if ($wizard_style)
                echo _("Step 2: General Information about your IdP");
            else
                printf(_("Editing IdP information for '%s'"),$my_inst->name);
            ?>
        </h1>
        <div class='infobox'>
            <h2><?php echo _("General Institution Properties"); ?></h2>
            <table>
                <tr>
                    <td><?php echo _("Country:");?></td>
                    <td><strong><?php echo Federation::name($my_inst->federation);?></strong></td>
                </tr>
                <tr>
                    <td><?php echo _("Institution name:");?></td>
                    <td><strong><?php echo $my_inst->name;?></strong></td>
                </tr>
                <?php echo infoblock($idpoptions, "general", "IdP");?>
            </table>
        </div>
        <?php
        echo "<form enctype='multipart/form-data' action='idp_wizard.php?inst_id=$my_inst->identifier".($wizard_style ? "&wizard=true" : "")."$lang_string' method='post'>
              <input type='hidden' name='MAX_FILE_SIZE' value='".Config::$MAX_UPLOAD_SIZE."'>";

        if ($wizard_style)
            echo "<p>"._("Hello, newcomer. Your institution is new to us. This wizard will ask
                 you several questions about your IdP, so that we can generate beautiful
                 profiles for you in the end. All of the information below is optional,
                 but it is important to fill out as many fields as possible for the benefit
                 of your end users.")."</p>";
        ?>
        <fieldset class="option_container">
            <legend><strong><?php echo _("General Information"); ?></strong></legend>
            <?php
            if ($wizard_style) {
                echo "<p>"._("This is the place where you can describe your institution in a fine-grained way.
                        The solicited information is used as follows:")."</p>
                      <ul>
                         <li>"._("<strong>Logo</strong>: When you submit a logo, we will embed this logo into all installers
                where a custom logo is possible. We accept any image format, but for best results, we suggest SVG. If you don't upload a logo, we will use the generic eduroam logo instead.")."</li>
                         <li>"._("<strong>Terms of Use</strong>: Some installers support displaying text to the user during installation time. If so, we will make that happen if you upload an RTF file or plain text file to display.")."</li>
                         <li>"._("<strong>Additional SSIDs</strong>: We will always configure the SSID 'eduroam' for WPA2/AES (and WPA/TKIP if the device supports multiple profiles). If you want to have more SSIDs included, you can specify them here. By using the '(with WPA/TKIP)' option you can specify that we should include legacy support for WPA/TKIP where possible.")."</li>
                </ul>";
            }
            ?>
            <table id="expandable_inst_options">
                <?php
                if ($edit_mode) $prepopulate=$idpoptions;
                $class="general";
                include "inc/add_option.php";
                ?>
            </table>
            <button type='button' class='newoption' onclick='addDefaultInstOptions("<?php echo $lang_string;?>")'><?php echo _("Add new option"); ?></button>
        </fieldset>
        <?php
        $part = "body";
        $additional = FALSE;

        foreach ($idpoptions as $optionname => $optionvalue) {
            if ($optionvalue['name'] == "general:geo_coordinates") {
                $additional = TRUE;
            }
        }
        include "inc/geo_widget.php";
        ?>
        <fieldset class="option_container">
            <legend><strong><?php echo _("Helpdesk Details for all users");?></strong></legend>
            <?php
            if ($wizard_style)
                echo "<p>"._("If your IdP provides a helpdesk for its users, it would be nice if you
            would tell us the pointers to this helpdesk. Some site installers might
            be able to signal this information to the user if he gets stuck.")."</p>
        <p>"._("If you enter a value here, it will be added to the site installers for
            all your users, and will be displayed on the download page. If you
            operate separate helpdesks for different user groups (we call this
            'profiles'), or operate no help desk at all (shame on you!), you can
            also leave any of these fields empty and optionally specify
            per-profile helpdesk information later in this wizard.")."</p>";
            ?>

            <table id="expandable_support_options">
                <?php
                if ($edit_mode) $prepopulate=$idpoptions;
                $class="support";
                include "inc/add_option.php";
                ?>
            </table>
            <button type='button' class='newoption' onclick='addDefaultSupportOptions("<?php echo $lang_string;?>")'><?php echo _("Add new option"); ?></button></fieldset>
        <fieldset class="option_container">
            <legend><strong><?php echo _("EAP details for all users");?></strong></legend>
            <?php
            if ($wizard_style)
                echo "<p>"._("Most EAP methods need server-side authentication details, like the CA
            certificate and/or server name(s) of your authentication servers. If
            all the EAP methods you support work with the same CA and or Common Names
            of servers, you can enter them here and they will be added as trust
            anchors in all profiles. If the details differ per profile or per EAP-type,
            you can also enter them in the individual profiles later.")."</p>
        <p>"._("<strong>Note well: </strong>The server-side validation is a cornerstone of eduroam; without it, users are
            subject to man-in-the-middle attacks! We will not generate site installers without Trusted CA anchors and server names.")."</p>";
            ?>
            <table id="expandable_eapserver_options">
                <?php
                $class="eap";
                if ($edit_mode) $prepopulate=$idpoptions;
                include "inc/add_option.php";
                ?>
            </table>
            <button type='button' class='newoption' onclick='addDefaultEapServerOptions("<?php echo $lang_string;?>")'><?php echo _("Add new option"); ?></button>
        </fieldset>
        <?php
        if ($wizard_style) {
            echo "<p>"._("When you are sure that everything is correct, please click on <button type='submit' name='submitbutton' value='continue'>Continue ...</button>")."</p></form>";
        } else {
            echo "<div><button type='button' name='abortbutton' value='abort' onclick='javascript:window.location = \"dashboard.php?inst_id=$my_inst->identifier$lang_string\"'>"._("Discard changes")."</button> <button type='submit' name='submitbutton' value='save'>"._("Save data")."</button></div></form>";
        }
        ?>
        <?php include "inc/admin_footer.php"; ?>
    </body>
</html>
