<?php
/**
 * This file contains the installer for iOS devices and Apple 10.7 Lion
 *
 *
 * @author Stefan Winter <stefan.winter@restena.lu>
 * @package Developer
 */

/**
 * 
 */
require_once('Device.php');
require_once('Helper.php');

// set_locale("devices");

/**
 * This is the main implementation class of the module
 *
 * The class should only define one public method: writeInstaller.
 *
 * All other methods and properties should be private. This example sets zipInstaller method to protected, so that it can be seen in the documentation.
 *
 * @package Developer
 */
class Device_mobileconfig extends DeviceConfig {

  /**
   * this array holds the list of EAP methods supported by this device
   */
   // public static $my_eap_methods = array(EAP::$TLS, EAP::$PEAP_MSCHAP2, EAP::$TTLS_PAP, EAP::$FAST_GTC);
    final public function __construct() {
      $this->supportedEapMethods  = array(EAP::$TLS, EAP::$PEAP_MSCHAP2, EAP::$TTLS_PAP);
      debug(4,"This device supports the following EAP methods: ");
      debug(4,$this->supportedEapMethods);
    }


   public static $my_eap_methods = array(array("OUTER" => TLS, "INNER" => NONE), array("OUTER" => PEAP, "INNER" => MSCHAPv2), array("OUTER" => TTLS, "INNER" => NONE));

   private $massaged_inst;
   private $massaged_profile;
   private $massaged_country;
   private $lang;

   /**
   * prepare a zip archive containing files and settings which normally would be used inside the module to produce an installer
   *
   * {@source}
   */
   public function writeInstaller() {
   /** run innitial setup
      this will:
       - create the temporary directory and save its path as $this->FPATH
       - process the CA certificates and store results in $this->attributes['internal:CAs'][0]
            $this->attributes['internal:CAs'][0] is an array of processed CA certificates
            a processed certifincate is an array 
               'pem' points to pem feromat certificate
               'der' points to der format certificate
               'md5' points to md5 fingerprint
               'sha1' points to sha1 fingerprint
               'name' points to the certificate subject
       - save the info_file (if exists) and put the name in $this->attributes['internal:info_file_name'][0]
   */
       $dom = textdomain(NULL);
      textdomain("devices");
    $this->supportedEapMethods = Device_mobileconfig::$my_eap_methods;

    debug(4,"mobileconfig Module Installer start\n");
//    $this->setup($profile);
    $this->massaged_inst = preg_replace('/ +/','_',$this->attributes['internal:inst_name'][0]);
    $this->massaged_profile = preg_replace('/ +/','_',$this->attributes['internal:profile_name'][0]);
    $this->massaged_country = strtolower($this->attributes['internal:country'][0]);
    $this->lang = preg_replace('/\..+/','',setlocale(LC_ALL,"0"));

    if (isset($this->attributes['internal:use_anon_outer']) && $this->attributes['internal:use_anon_outer'][0] == "1" && isset($this->attributes['internal:realm']))
            $use_realm = $this->attributes['internal:realm'][0];
    else
            $use_realm = 0;

    $filename = $this->attributes['internal:inst_name'][0]."-".$this->attributes['internal:profile_name'][0].".mobileconfig";
    $filename = preg_replace('/ +/','_',$filename);
    $xml_f = fopen($filename,'w');


    $ssid_list = $this->attributes['general:SSID'];
    $server_names = $this->attributes['eap:server_name'];
    $uuid_list = $this->list_ca_uuids($this->attributes['internal:CAs'][0]);
    $eap_type = $this->selected_eap;

    fwrite($xml_f,"<?xml version=\"1.0\" encoding=\"utf-8\"?>
<!DOCTYPE plist PUBLIC \"-//Apple Computer//DTD PLIST 1.0//EN\"
\"http://www.apple.com/DTDs/PropertyList-1.0.dtd\">
<plist version=\"1.0\">
   <dict>
      <key>PayloadContent</key>
         <array>");

    fwrite($xml_f,$this->all_wlan($ssid_list,$server_names,$uuid_list,$eap_type, $use_realm));

    fwrite($xml_f,$this->all_ca($this->attributes['internal:CAs'][0]));

    fwrite($xml_f,"
         </array>
      <key>PayloadDescription</key>
         <string>".sprintf(_("iPhone profile '%s' of '%s' - provided by eduroam.org"),$this->attributes['internal:profile_name'][0],$this->attributes['internal:inst_name'][0])."</string>
      <key>PayloadDisplayName</key>
         <string>".$this->attributes['internal:profile_name'][0]."</string>
      <key>PayloadIdentifier</key>
         <string>".Device_mobileconfig::$IPHONE_PAYLOAD_PREFIX.".$this->massaged_country.$this->massaged_inst.$this->massaged_profile.$this->lang</string>
      <key>PayloadOrganization</key>
         <string>".$this->attributes['internal:inst_name'][0]."</string>
      <key>PayloadType</key>
         <string>Configuration</string>
      <key>PayloadUUID</key>
         <string>".uuid()."</string>
      <key>PayloadVersion</key>
         <integer>1</integer>
   </dict>
</plist>");

    fclose($xml_f);
   if($this->sign) {
      $e = 'signed-'.$filename;
      $o = system($this->sign." $filename $e > /dev/null");
   }
   else
      $e = $filename;

   textdomain($dom);
    return $e;
   }

   static private $IPHONE_PAYLOAD_PREFIX = "org.eduroam";

   private function list_ca_uuids($ca_array) {
       $retval = array();
       foreach ($ca_array as $ca)
           $retval[] = $ca['uuid'];
       return $retval;
   }

   private function all_wlan ($SSID_list, $server_list, $CA_UUID_list, $eap_type, $realm = 0) {
       $SSID_list[] = "eduroam";
       $retval = "";
       $serial = 0;
       foreach ($SSID_list as $SSID) {
            $retval .= "
            <dict>
               <key>EAPClientConfiguration</key>
                  <dict>
                      <key>AcceptEAPTypes</key>
                         <array>
                            <integer>".$eap_type['OUTER']."</integer>
                         </array>
                      <key>EAPFASTProvisionPAC</key>
                            <true />
                      <key>EAPFASTUsePAC</key>
                            <true />
                      <key>EAPFastProvisionPACAnonymously</key>
                            <false />
                      <key>OneTimeUserPassword</key>
                            <false />
";
         if ($realm !== 0)
                     $retval .= "<key>OuterIdentity</key>
                                    <string>@$realm</string>
";
         $retval .=             "<key>PayloadCertificateAnchorUUID</key>
                         <array>";
         foreach ($CA_UUID_list as $uuid)
            $retval .= "
<string>$uuid</string>";
         $retval .= "
                         </array>
                      <key>TLSAllowTrustExceptions</key>
                         <false />
                      <key>TLSTrustedServerCommonNames</key>
                         <array>";
         foreach ($server_list as $cn)
            $retval .= "
<string>$cn</string>";
         $retval .= "
                         </array>
                      <key>TTLSInnerAuthentication</key>
                         <string>PAP</string>
                   </dict>
               <key>EncryptionType</key>
                  <string>WPA</string>
               <key>HIDDEN_NETWORK</key>
                  <false />
               <key>PayloadDescription</key>
                  <string>"._("eduroam configuration for network name $SSID")."</string>
               <key>PayloadDisplayName</key>
                  <string>".sprintf(_("SSID %s"),$SSID)."</string>
               <key>PayloadIdentifier</key>
                  <string>".Device_mobileconfig::$IPHONE_PAYLOAD_PREFIX.".$this->massaged_country.$this->massaged_inst.$this->massaged_profile.$this->lang.wifi.$serial</string>
               <key>PayloadOrganization</key>
                  <string>eduroam.org</string>
               <key>PayloadType</key>
                  <string>com.apple.wifi.managed</string>
               <key>PayloadUUID</key>
                  <string>".uuid()."</string>
               <key>PayloadVersion</key>
                  <integer>1</integer>
               <key>SSID_STR</key>
                  <string>$SSID</string>
            </dict>";
         $serial = $serial + 1;
       }
       return $retval;
   }

   private function all_ca ($ca_array) {
       $retval = "";
       $i = 0;
       foreach ($ca_array as $ca) {
           $retval .= $this->ca_blob($ca['uuid'], $ca['pem'],$i);
           $i = $i+1;
       }
       return $retval;
   }
       private function ca_blob($uuid, $pem, $serial) {
           // cut lines with CERTIFICATE
           $pem = preg_replace('/-----BEGIN CERTIFICATE-----/','', $pem);
           $pem = preg_replace('/-----END CERTIFICATE-----/','', $pem);
           $pem = trim($pem);
           //return print_r($result);
	$stream ="
            <dict>
               <key>PayloadCertificateFileName</key>
               <string>$uuid.der</string>
               <key>PayloadContent</key>
               <data>
".$pem."</data>
               <key>PayloadDescription</key>
               <string>"._("Your Identity Providers Certification Authority")."</string>
               <key>PayloadDisplayName</key>
               <string>"._("Identity Provider's CA")."</string>
               <key>PayloadIdentifier</key>
               <string>".Device_mobileconfig::$IPHONE_PAYLOAD_PREFIX.".$this->massaged_inst.$this->massaged_profile.credential.$serial</string>
               <key>PayloadOrganization</key>
               <string>eduroam.org</string>
               <key>PayloadType</key>
               <string>com.apple.security.root</string>
               <key>PayloadUUID</key><string>".$uuid."</string>
               <key>PayloadVersion</key>
               <integer>1</integer>
            </dict>";

	return $stream;
}


}
