<?php
/**
 * This file contains the Devices class.
 *
 * @package ModuleWriting
 */
/**
 * The Devices class holds a list of all devices the CAT knows about
 * 
 * @author Tomasz Wolniewicz <twoln@umk.pl>
 * 
 * @license http://ticker.eduroam.lu/cat/LICENSE
 * 
 * @package ModuleWriting
 */
class Devices{

/**
 * Each device is defined as a three-element array within this array
 *
 * - 'display' is the name shown on the GUI button
 * - 'directory' is the subdirectory of devices directory, where the device module resides
 * - 'module' is the name of the module class, the same name with .php added will be used as the name of the main include file for the module
 * - 'sign' if defined points to an external program which will sign a file. The first argument of this program must be the
 * input file name, the second - the signed file filename. 
 *
 * @example devices/devices.php file listing
 * @return array the device modules
 */
public static function listDevices() {
    return array(
 'vista'=>array(
   'display'=>_("MS Windows Vista and newer"),
   'directory'=>'ms_vista_plus',
   'module'=>'Vista7',
   /* 'sign'=>'/opt/local/GN3/CAT/ms_windows_sign' */
   ),
	
 'xp'=>array(
   'display'=>_("MS Windows XP SP3"),
   'directory'=>'ms_xp',
   'module'=>'WindowsXP',
   /* 'sign'=>'/opt/local/GN3/CAT/ms_windows_sign'*/
   ),
	
 'mobileconfig'=>array(
    'display'=>_("Apple mobile devices and Mac OS X Lion"),
    'directory'=>'apple_mobileconfig',
    'module'=>'mobileconfig',
    /* 'sign'=>'/opt/local/GN3/CAT/mobileconfig_sign' */
    ),

 'welcomeletter'=>array(
    'display'=>_("Welcome Letter"),
    'directory'=>'welcomeletter',
    'module'=>'welcomeletter'
   ),
    
 'test'=>array(
    'display'=>_("Test"),
    'directory'=>'test_module',
    'module'=>'TestModule'
   ),
);
}
}
?>
