<?php
require_once(dirname(dirname(dirname(dirname(__FILE__))))."/config/_config.php");
include_once("IdP.php");
include_once("Profile.php");
include_once("Helper.php");
include_once("common.inc.php");

if (isset($_GET['lang'])) {
    $ourlocale = set_locale("web_admin",$_GET['lang']);
    $lang_string = "&lang=".$_GET['lang'];
}
else {
    $ourlocale = set_locale("web_admin");
    $lang_string="";
}

header("Content-Type:text/html;charset=utf-8");
$my_inst = new IdP($_GET['inst_id']);

if (! $my_inst instanceof IdP) {
    echo "<p>"._("The page was called with a non-existing IdP identifier. This is not supposed to happen!")."</p>";
    exit(1);
}

$my_profile = new Profile($_GET['profile_id']);

if (! $my_profile instanceof Profile) {
    echo "<p>"._("The page was called with a non-existing profile identifier. This is not supposed to happen!")."</p>";
    exit(1);
}

function prepareCheck($profile) {
    $attribs = $profile->getCollapsedAttributes();
    // these are still arrays
    $cas = $attribs['eap:ca_file'];
    $names = $attribs['eap:server_name'];
    // store all CAs in temp files
    // does this really work, all in one file?
    $name = dirname(dirname(__FILE__))."/".Config::$TMP_BASE_PATH.md5(time().rand());
    debug(4,"temp file for certificates: $name\n");
    if(! mkdir($name,0700, true)) {
        error("unable to create temporary directory: $name\n");
        exit;
    }
    chdir($name);
    $filename = "certs.pem";
    $handle = fopen($filename,"w");
    foreach ($cas as $ca) {
        // echo $ca;
        fwrite($handle, $ca);
    }
    // calculate common suffix of all host names
    $suffix = calculateCommonHostSuffix($names);
    // echo $suffix;

    return " -a $filename -x $suffix";
}

function evalResult($flow,$eap,$host) {
    $failimage = "<img class='icon' src='../resources/images/icons/Exclamation-orange-icon.png' alt='"._("Error!")."' title='"._("Error!")."'/>";
    $goodimage = "<img class='icon' src='../resources/images/icons/Checkmark-lg-icon.png' alt='"._("OK!")."' title='"._("OK!")."'/>";
    $result = array_pop($flow);
    if ($result == 2)
        return "<tr><td>$goodimage</td><td>".sprintf(_("<td style='vertical-align: top'><strong>Success:</strong></td><td style='vertical-align: top'><strong>%s</strong> from <strong>%s</strong>"),display_name($eap),$host['display_name'])."</td></tr>";
    else
        return "<tr><td>$failimage</td><td>".sprintf(_("<td style='vertical-align: top'><strong>Failure:</strong></td><td style='vertical-align: top'>Login with <strong>%s</strong> from <strong>%s</strong> failed!"),display_name($eap),$host['display_name'])."<br/>Resultcode: $result</td></tr>";
}

if ($my_profile->realm != "") { // doing the checks

    $eaps = $my_profile->getEapMethodsinOrderOfPreference(1);
    $anon_id = "";
    if ($my_profile->use_anon_outer) $anon_id = "-A @$my_profile->realm";
    echo "<table>";
    foreach ($eaps as $pref => $eap) {
        foreach (Config::$RADIUS_HOSTS as $host) {
            if ($eap['OUTER'] == PEAP) {
                // we need a username and password
                if (!isset($_POST['username']) && $_POST['username'] == "") {
                    echo "<tr><td>$failimage</td><td>".sprintf(_("Could not check <strong>%s</strong> from <strong>%s</strong> - no username submitted!"),display_name($eap), $host['display_name'])."</td></tr>";
                    continue;
                }
                if (!isset($_POST['password']) && $_POST['password'] == "") {
                    echo "<tr><td>$failimage</td><td>".sprintf(_("Could not check <strong>%s</strong> from <strong>%s</strong> - no password submitted!"),display_name($eap), $host['display_name'])."</td></tr>";
                    continue;
                }
                if ($eap['INNER'] != MSCHAP2) {
                    echo "<tr><td>$failimage</td><td>".sprintf(_("Sorry, we don't know how to handle the inner EAP method in <strong>%s</strong> from <strong>%s</strong>!"),display_name($eap), $host['display_name'])."</td></tr>";
                    continue;
                }

                $certopts = prepareCheck($my_profile);

                $cmdline = "rad_eap_test -c -H ".$host['ip']." -P 1812 -S ".$host['secret']." -M 22:44:66:CA:20:01 $anon_id -u ".$_POST['username']." -p ".$_POST['password']." -e PEAP -m WPA-EAP $certopts -t ".$host['timeout']." | grep 'RADIUS message:' | cut -d ' ' -f 3 | cut -d '=' -f 2";
                // debug(4,"Thorough reachability check: $cmdline.\n");
                $packetflow = array();
                exec($cmdline,$packetflow);

                echo evalResult($packetflow,$eap,$host);

            }
            else if ($eap['OUTER'] == TTLS) {
                // we need a username and password
                if (!isset($_POST['username']) && $_POST['username'] == "") {
                    echo "<tr><td>$failimage</td><td>".sprintf(_("Could not check <strong>%s</strong> from <strong>%s</strong> - no username submitted!"),display_name($eap), $host['display_name'])."</td></tr>";
                    continue;
                }
                if (!isset($_POST['password']) && $_POST['password'] == "") {
                    echo "<tr><td>$failimage</td><td>".sprintf(_("Could not check <strong>%s</strong> from <strong>%s</strong> - no password submitted!"),display_name($eap), $host['display_name'])."</td></tr>";
                    continue;
                }
                $inner_eap = "";
                if ($eap['INNER'] == MSCHAP2)
                    $inner_eap = "MSCHAPV2";
                if ($eap['INNER'] == PAP)
                    $inner_eap = "PAP";

                if ($inner_eap == "") {
                    echo "<tr><td>$failimage</td><td>".sprintf(_("Sorry, we don't know how to handle the inner EAP method in <strong>%s</strong> from <strong>%s</strong>!"),display_name($eap), $host['display_name'])."</td></tr>";
                    continue;
                }

                $certopts = prepareCheck($my_profile);

                $cmdline = "rad_eap_test -c -H ".$host['ip']." -P 1812 -S ".$host['secret']." -M 22:44:66:CA:20:01 $anon_id -u ".$_POST['username']." -p ".$_POST['password']." -e TTLS -2 $inner_eap -m WPA-EAP $certopts -t ".$host['timeout']." | grep 'RADIUS message:' | cut -d ' ' -f 3 | cut -d '=' -f 2";
                $packetflow = array();
                exec($cmdline,$packetflow);

                echo evalResult($packetflow,$eap,$host);

            }
            else if ($eap['OUTER'] == TLS) {
                // we need a certificate file
                // check TBD

                $cmdline = "rad_eap_test -c -H ".$host['ip']." -P 1812 -S ".$host['secret']." -M 22:44:66:CA:20:01 $anon_id -e TLS -m WPA-EAP -t ".$host['timeout']." | grep 'RADIUS message:' | cut -d ' ' -f 3 | cut -d '=' -f 2";
                echo "<tr><td>$failimage</td><td>INCOMPLETE I would execute this:<br/>$cmdline</td></tr>";
            }
            else
                echo "<tr><td>$failimage</td><td>".sprintf(_("Sorry... we don't know how to test the EAP type %s from %s"),display_name($eap),$host['display_name'])."</td></tr>";
        }
    }
    echo "</table>";

    /*                               <tr><td>Username:</td><td><input type='text' id='username' name='username'/></td></tr>";
                                <tr><td>Password:</td><td><input type='text' id='password' name='password'/></td></tr>";
                              <tr><td>Certificate file:</td><td><input type='file' id='cert' name='cert'/></td></tr>
                              <tr><td>Private key, if any:</td><td><input type='text' id='privkey' name='privkey'/></td></tr>";
    */

} else {
    echo "<p>$failimage"._("Your profile does not contain a valid realm name. We can only check the validity of a realm if you let us know the realm name. Please consider going to your profile properties and enter the realm name.")."</p>";
}
?>