<?php
require_once(dirname(dirname(dirname(__FILE__)))."/config/_config.php");
include_once("Federation.php");
include_once("IdP.php");
include_once("Profile.php");
include_once("Helper.php");
include_once("inc/common.inc.php");

if (isset($_GET['lang'])) {
    $ourlocale = set_locale("web_admin",$_GET['lang']);
    $lang_string = "&lang=".$_GET['lang'];
}
else {
    $ourlocale = set_locale("web_admin");
    $lang_string="";
}

// let's check if the inst handle actually exists in the DB

$my_inst = new IdP($_GET['inst_id']);
$my_profile = FALSE;

if (! $my_inst instanceof IdP) {
    echo "<p>"._("The page was called with a non-existing IdP identifier. This is not supposed to happen!")."</p>";
    exit(1);
}

if (isset($_GET['profile_id']) ) { // oh! We should edit an existing profile, not create a new one!
    $inst_profiles = $my_inst->listProfiles();
    foreach ($inst_profiles as $profile)
        if ($profile->identifier == $_GET['profile_id']) {
            $my_profile = $profile;
            $wizard_style = FALSE;
            $edit_mode = TRUE;
            $prefill_name = $my_profile->name;
            $anon_realm = $my_profile->getAnonymousIDSupport();
            $realm = $my_profile->getRealm();
            $prefill_methods = $my_profile->getEapMethodsinOrderOfPreference();
            $profile_options = $my_profile->getAttributes();
            break;
        }
    if ( $my_profile === FALSE) { // no profile <-> inst match!
        echo "<p>"._("The page was called with a Profile identifier, but there was no such profile in this institution!")."</p>";
        exit(1);
    }
} else {
    $wizard_style = TRUE;
    $edit_mode = FALSE;
    $prefill_name = "";
    $anon_realm = "";
    $realm = "";
    $prefill_methods = array();
    $profile_options = array();
}

header("Content-Type:text/html;charset=utf-8");
$idpoptions = $my_inst->getAttributes();
?>
<!DOCTYPE html>
<html>
    <head>
        <meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
        <meta http-equiv="content-language" content="<?php echo $ourlocale;?>">
        <link rel="stylesheet" type="text/css" href="../resources/css/cat.css" />
        <script src="js/option_expand.js" type="text/javascript"></script>
        <!-- JQuery --> 
        <script type="text/javascript" src="../external/jquery/jquery.js"></script> 
        <script type="text/javascript" src="../external/jquery/jquery-ui.js"></script> 
        <!-- EAP sorting code -->
        <style>
            li.eap1 {list-style-type: none;}
            ol.eapmethods { list-style-position: inside; margin: 0; padding: 0px; padding-top: 20px; padding-bottom: 0px; width: 20em; }
            ol.eapmethods li{
                background: #CCF;
                border-left-style: inset;
                border-left-width: 1px;
                border-left-color: #8BBACB;
                border-top-style: inset;
                border-top-width: 1px;
                border-top-color: #8BBACB;
                border-right-style: outset;
                border-right-width: 2px;
                border-right-color: #043D52;
                border-bottom-style: outset;
                border-bottom-width: 2px;
                border-bottom-color: #043D52;
                border-radius: 6px;
                box-shadow: 4px 4px 4px #888888;
                background-image:url('../resources/images/icons/strzalka5.png');
                background-repeat:no-repeat;
                background-position:95% 50%;
                margin: 2px 0px 2px 0px;
                padding: 3px;
                padding-left: 1em;
                padding-right: 0px;
            }

            table.eaptable td {
                background:#F0F0F0;
            }

            table.eaptable th {
                background:#F0F0F0;
            }

            #eap_bottom_row td {
                border-top-color: #888;
                border-top-style: solid;
                border-top-width: 2px;
            }

            #eap_bottom_row th {
                border-top-color: #888;
                border-top-style: solid;
                border-top-width: 2px;
            }

        </style>
        <script>
            $(function() {
                $( "#sortable1, #sortable2" ).sortable({
                    connectWith: "ol.eapmethods",
                    tolerance: 'pointer',
                    out: function(event, ui) {
                        ui.item.toggleClass("eap1");
                    },
                    stop: function(event, ui) {
                        $(".eapm").removeAttr('value');
                        $(".eapmv").removeAttr('value');
                        $( "#sortable1" ).children().each(function(index){
                            i = index + 1;
                            v = $(this).html();
                            $("#EAP-"+v).val(v);
                            $("#EAP-"+v+"-priority").val(i);
                        });
                    }
                }).disableSelection();
            });
        </script>
        <!-- EAP sorting code end -->
        <title>
            <?php
            if ($wizard_style)
                printf(_("%s: IdP Enrollment Wizard (Step 3)"),Config::$PRODUCT_NAME);
            else
                printf(_("%s: Edit Profile '%s' ..."),Config::$PRODUCT_NAME,$prefill_name);
            ?>
        </title>
    </head>
    <body>
        <?php
        include "inc/admin_header.php";
        // If we got here, and have submitted POST data, then we apparently have just run successfully through
        // profile generation, and the user wants to add another one. Tell him so.
        if ( $_POST['submitbutton'] == "submitted-more" )
            echo "<p><font color='#7fff7f'>"._("Profile generated successfully!")."</font></p>";
        ?>
        <h1>
            <?php
            if ($wizard_style)
                echo _("Step 3: Defining a user group profile");
            else
                printf(_("Edit profile '%s' ..."), $prefill_name);
            ?>
        </h1>
        <div class='infobox'>
            <h2><?php echo _("General Institution Details");?></h2>
            <table>
                <tr>
                    <td>
                        <?php echo ""._("Country:")?>
                    </td>
                    <td>
                        <strong><?php echo Federation::name($my_inst->federation);?></strong>
                    </td>
                </tr>
                <tr>
                    <td>
                        <?php echo _("Institution name:");?>
                    </td>
                    <td>
                        <strong><?php echo $my_inst->name;?></strong>
                    </td>
                </tr>
                <?php echo infoblock($idpoptions, "general", "IdP");?>
            </table>
        </div>
        <div class="infobox">
            <h2><?php echo _("Global Helpdesk Details");?></h2>
            <table><?php echo infoblock($idpoptions, "support", "IdP");?></table>
        </div>
        <div class='infobox'>
            <h2><?php echo _("Global EAP Options");?></h2>
            <table><?php echo infoblock($idpoptions, "eap", "IdP");?></table>
        </div>
        <?php
        echo "<form enctype='multipart/form-data' action='profile_submission_result.php?inst_id=$my_inst->identifier".($edit_mode ? "&profile_id=".$my_profile->identifier : "")."$lang_string' method='post'>
                <input type='hidden' name='MAX_FILE_SIZE' value='".Config::$MAX_UPLOAD_SIZE."'>";
        ?>
        <fieldset class="option_container">
            <legend>
                <strong><?php echo _("General Profile properties");?></strong>
            </legend>
            <?php
            if ($wizard_style)
                echo "<p>"._("We will now define a profile for your user group(s).  You can add as many profiles as you like by choosing the appropriate button on the end of the page. After we are done, the wizard is finished and you will be taken to the main IdP administration page.")."</p>";
            ?>
            <h3><?php echo _("Profile Name and RADIUS realm");?></h3>
            <table>
            <?php
            if ($wizard_style)
                echo "<tr><td colspan='2'>"._("First of all we need a name for the profile. This will be displayed to end users, so you may want to choose a descriptive name like 'Professors', 'Students of the Faculty of Bioscience', etc.")."</td></tr>";
            ?>
            <label>
                <tr>
                    <td><?php echo _("Profile Name:")."</td><td><input name='profile_name' title='Profile Name' value='$prefill_name'/></td>"; ?></td>
                </tr>
            </label>
            <br/>
            <?php
            if ($wizard_style)
                echo "<tr><td colspan='2'>"._("You can tell us your RADIUS realm.");
            if (Config::$CONSORTIUM_NAME == "eduroam") printf(_("This is useful if you want to use the sanity check module later, which tests reachability of your realm in the %s infrastructure."),CONFIG::$CONSORTIUM_NAME);
            echo _("It is required to enter the realm name if you want to support anonymous outer identities (see below).")."</td></tr>";
            ?>
            <label>
                <tr>
                    <td>
                <?php
                echo _("Realm:")."</td><td><input id='realm' name='realm' value='$realm' onkeyup='
                                 if (this.value.length > 0)
                                      { this.form.elements[\"anon_support\"].removeAttribute(\"disabled\");
                                        document.getElementById(\"anon_support_label\").removeAttribute(\"style\");
                                      } else
                                      { this.form.elements[\"anon_support\"].checked = false;
                                        this.form.elements[\"anon_support\"].setAttribute(\"disabled\", \"disabled\");
                                        document.getElementById(\"anon_support_label\").setAttribute(\"style\", \"color:#999999\");
                                      };'/>";?>
            </td>
                </tr>
            </label>
            </table>
            <h3><?php echo _("Anonymity Support");?></h3>

            <?php
            if ($wizard_style)
                echo "<p>"._("Some installers support a feature called 'Anonymous outer identity'. If you don't know what this is, please read <a href='anonymous.html'>this article</a>. Do you want us to generate installers with anonymous outer identities where available? You need to fill out the 'Realm' field above for this to work.")."</p>";
            ?>
            <p>
                <label>
                    <?php
                    echo "<span id='anon_support_label' style='".($realm == "" ? "color:#999999" : "" )."'>"._("Enable Anonymous Outer Identity:")."</span>
                             <input type='checkbox' ".($anon_realm != "" ? "checked" : "" ).($realm == "" ? "disabled" : "" )." name='anon_support' />";
                    ?>
                </label>
            </p>

            <h3><?php echo _("Installer Download Location");?></h3>

            <?php
            if ($wizard_style)
                echo "<p>"._("The CAT has a download area for end users. There, they will, for example, learn about the support pointers you entered earlier. The CAT can also immediately offer the installers for the profile for download. If you don't want that, you can instead enter a web site location where you want your users to be redirected to. You, as the administrator, can still download the profiles to place them on that page (see the 'Compatibility Matrix' button on the dashboard).")."</p>";
            ?>
            <p>
                <label>
                    <?php
                    echo "<span id='redirect_label' style='".($realm == "" ? "color:#999999" : "" )."'>"._("Redirect end users to own web page:")."</span>
                          <input type='checkbox'  name='redirect' />
                          <input type='text' name='redirect_target' />";
                    ?>
                </label>
            </p>

        </fieldset>
        <fieldset class="option_container">
            <legend><strong><?php echo _("Supported EAP types");?></strong></legend>
            <?php
            if ($wizard_style)
                echo "<p>"._("Now, we need to know which EAP types your IdP supports. If you support multiple EAP types, you can assign every type a priority (1=highest). This tool will always generate an automatic installer for the EAP type with the highest priority; only if the user's device can't use that EAP type, we will use an EAP type further down in the list.")."</p>";
            ?>
            <?php
            function priority ($eap_type, $isenabled, $priority) {
                echo "<td><select id='$eap_type-priority' name='$eap_type-priority' ".(!$isenabled ? "disabled='disabled'" : "").">";
                for ($a=1; $a<7; $a=$a+1)
                    echo "<option id='$eap_type-$a' value='$a' ".( $isenabled && $a == $priority ? "selected" : "" ).">$a</option>";
                echo "</select></td>";
            }

            function inherited_options ($idpwideoptions, $eap_type, $is_visible) {
                echo "<td><div style='".(!$is_visible ? "visibility:hidden" : "")."' class='inheritedoptions' id='$eap_type-inherited-global'>";

                $eapoptions = array();

                foreach ($idpwideoptions as $option)
                    if ( $option['level'] == "IdP" && preg_match('/^eap/', $option['name']) )
                        $eapoptions[] = $option['name'];

                $eapoptions = array_count_values($eapoptions);

                if (count ( $eapoptions ) > 0 ) {
                    echo "<strong>"._("EAP options inherited from Global level:")."</strong><br />";
                    foreach ($eapoptions as $optionname => $count)
                        printf(_("(%dx) %s")."<br />",$count,display_name($optionname));
                }

                echo "</div></td>";
            }

            $methods = EAP::listKnownEAPTypes();
            ?>

            <?php
// new EAP sorting code  

            foreach ($methods as $a) {
                $display = display_name($a);
                $enabled = FALSE;
                if ($edit_mode)
                    foreach ($prefill_methods as $prio => $value) {
                        if  (display_name($a) == display_name($value)) {
                            $enabled = TRUE;
                            $countactive=$prio+1;
                        }
                    }
            } ?>
            <div>
                <table border='0'>
                    <tr>
                        <th style="vertical-align:top; padding:1em">
                            <?php echo _('Supported EAP types for this profile');?>
                        </th>
                        <td>
                            <ol id="sortable1" class="eapmethods">
                                <?php
                                $D = array();
                                foreach ($prefill_methods as $prio => $value) {
                                    print '<li>'.display_name($value)."</li>\n";
                                    $D[display_name($value)] = $prio;
                                }?>
                            </ol>
                        </td>
                        <td rowspan=3 style="text-align:center; width:12em; padding:1em">
                            <?php echo _('Use "drag &amp; drop" to mark an EAP method as supported. Prioritisation is done automatically, depending on where you "drop" the method.');?>
                        </td>
                    </tr>
                    <tr id="eap_bottom_row">
                        <td colspan="2"> </td>
                    </tr>
                    <tr>
                        <th style="vertical-align:top; padding:1em">
                            <?php echo _('Unsupported EAP types');?>
                        </th>
                        <td style="vertical-align:top">
                            <ol id="sortable2" class="eapmethods">
                                <?php
                                foreach ($methods as $a) {
                                    $display = display_name($a);
                                    if  (! isset($D[display_name($a)]))
                                        print '<li class="eap1">'.display_name($a)."</li>\n";
                                }
                                ?>
                            </ol>
                        </td>
                    </tr>
                </table>
            </div>
            <?php
            foreach ($methods as $a) {
                $display = display_name($a);
                $v = isset($D[$display]) ? $D[$display] : '';
                print '<input type="hidden" class="eapm" name="'.$display.'" id="EAP-'.$display.'" value="'.$display.'">';
                print '<input type="hidden" class="eapmv" name="'.$display.'-priority" id="EAP-'.$display.'-priority" value="'.$v.'">';
            }?>
            <br clear="both" />
        </fieldset>
        <fieldset class="option_container" id="helpdesk_override">
            <legend><strong><?php echo _("Helpdesk Details for this profile");?></strong></legend>
            <p>
                <?php echo _("If you specify an option here, it will override the global option(s) with the same name, if any.");?>
            </p>
            <table id="expandable_support_options">
                <?php
                $class="support";
                if ($edit_mode) {
                    $prepopulate=array();
                    $existing_attribs=$my_profile->getAttributes();
                    foreach ($existing_attribs as $existing_attribute)
                        if ( $existing_attribute['level'] == "Profile" ) $prepopulate[] = $existing_attribute;
                }
                include "inc/add_option.php";
                ?>
            </table>
            <button type='button' class='newoption' onclick='addDefaultSupportOptions("<?php echo $lang_string;?>")'><?php echo _("Add new option"); ?></button>
        </fieldset>
        <fieldset class="option_container" id="eap_override">
            <legend><strong><?php echo _("EAP Details for this profile");?></strong></legend>
            <p>
                <?php echo _("If you specify an option here, it will override the global option(s) with the same name, if any.");?>
            </p>
            <table id="expandable_eapserver_options">
                <?php
                $class="eap";
                include "inc/add_option.php";
                ?>
            </table>
            <button type='button' class='newoption' onclick='addDefaultEapServerOptions("<?php echo $lang_string;?>")'><?php echo _("Add new option"); ?></button>
        </fieldset>
        <?php
        if ($wizard_style)
            echo "<p>"._("When you are sure that everything is correct, please click on
                 <button type='submit' name='submitbutton' value='submitted-dashboard'>Continue to dashboard</button>")."</p>
                 <p>"._("If you want to set up another profile after this one, please click on
                 <button type='submit' name='submitbutton' value='submitted-more'>Add another profile</button>")."</p></form>";
        else
            echo "<p><button type='submit' name='submitbutton' value='submitted-dashboard'>"._("Save data")."</button><button type='button' class='delete' name='abortbutton' value='abort' onclick='javascript:window.location = \"dashboard.php?inst_id=$my_inst->identifier$lang_string\"'>"._("Discard changes")."</button></p></form>";

        include "inc/admin_footer.php";
        ?>
    </body>
</html>
